@@ -4,6 +4,6 @@
 
 #include <map>
 
-const std::map<int, const char *> &GetAACEncoderBitrateMap();
+const std::map<int, std::string> &GetAACEncoderBitrateMap();
 const char *GetAACEncoderForBitrate(int bitrate);
 int FindClosestAvailableAACBitrate(int bitrate);
