@@ -8,7 +8,27 @@
 #include "auth-oauth.hpp"
 
 #ifdef BROWSER_AVAILABLE
-class BrowserDock;
+#include "window-dock-browser.hpp"
+#include "lineedit-autoresize.hpp"
+#include <QHBoxLayout>
+class YoutubeChatDock : public BrowserDock {
+	Q_OBJECT
+
+private:
+	std::string apiChatId;
+	LineEditAutoResize *lineEdit;
+	QPushButton *sendButton;
+	QHBoxLayout *chatLayout;
+
+public:
+	void SetWidget(QCefWidget *widget_);
+	void SetApiChatId(const std::string &id);
+
+private slots:
+	void SendChatMessage();
+	void ShowErrorMessage(const QString &error);
+	void EnableChatInput();
+};
 #endif
 
 inline const std::vector<Auth::Def> youtubeServices = {
@@ -20,11 +40,10 @@ class YoutubeAuth : public OAuthStreamKey {
 	Q_OBJECT
 
 	bool uiLoaded = false;
-	std::mt19937 randomSeed;
 	std::string section;
 
 #ifdef BROWSER_AVAILABLE
-	QSharedPointer<BrowserDock> chat;
+	QSharedPointer<YoutubeChatDock> chat;
 	QSharedPointer<QAction> chatMenu;
 #endif
 
@@ -38,7 +57,7 @@ class YoutubeAuth : public OAuthStreamKey {
 public:
 	YoutubeAuth(const Def &d);
 
-	void SetChatId(QString &chat_id);
+	void SetChatId(const QString &chat_id, const std::string &api_chat_id);
 	void ResetChat();
 
 	static std::shared_ptr<Auth> Login(QWidget *parent,
