@@ -12,55 +12,19 @@
 #include "log-viewer.hpp"
 #include "qt-wrappers.hpp"
 
-OBSLogViewer::OBSLogViewer(QWidget *parent) : QDialog(parent)
+OBSLogViewer::OBSLogViewer(QWidget *parent)
+	: QDialog(parent), ui(new Ui::OBSLogViewer)
 {
 	setWindowFlags(windowFlags() & Qt::WindowMaximizeButtonHint &
 		       ~Qt::WindowContextHelpButtonHint);
 	setAttribute(Qt::WA_DeleteOnClose);
 
-	QVBoxLayout *layout = new QVBoxLayout();
-	layout->setContentsMargins(0, 0, 0, 0);
-
-	const QFont fixedFont =
-		QFontDatabase::systemFont(QFontDatabase::FixedFont);
-
-	textArea = new QTextEdit();
-	textArea->setReadOnly(true);
-	textArea->setFont(fixedFont);
-
-	QHBoxLayout *buttonLayout = new QHBoxLayout();
-	QPushButton *clearButton = new QPushButton(QTStr("Clear"));
-	connect(clearButton, &QPushButton::clicked, this,
-		&OBSLogViewer::ClearText);
-	QPushButton *openButton = new QPushButton(QTStr("OpenFile"));
-	connect(openButton, &QPushButton::clicked, this,
-		&OBSLogViewer::OpenFile);
-	QPushButton *closeButton = new QPushButton(QTStr("Close"));
-	connect(closeButton, &QPushButton::clicked, this, &QDialog::close);
+	ui->setupUi(this);
 
 	bool showLogViewerOnStartup = config_get_bool(
 		App()->GlobalConfig(), "LogViewer", "ShowLogStartup");
 
-	QCheckBox *showStartup = new QCheckBox(QTStr("ShowOnStartup"));
-	showStartup->setChecked(showLogViewerOnStartup);
-	connect(showStartup, SIGNAL(toggled(bool)), this,
-		SLOT(ToggleShowStartup(bool)));
-
-	buttonLayout->addSpacing(10);
-	buttonLayout->addWidget(showStartup);
-	buttonLayout->addStretch();
-	buttonLayout->addWidget(openButton);
-	buttonLayout->addWidget(clearButton);
-	buttonLayout->addWidget(closeButton);
-	buttonLayout->addSpacing(10);
-	buttonLayout->setContentsMargins(0, 0, 0, 4);
-
-	layout->addWidget(textArea);
-	layout->addLayout(buttonLayout);
-	setLayout(layout);
-
-	setWindowTitle(QTStr("LogViewer"));
-	resize(800, 300);
+	ui->showStartup->setChecked(showLogViewerOnStartup);
 
 	const char *geom = config_get_string(App()->GlobalConfig(), "LogViewer",
 					     "geometry");
@@ -79,7 +43,7 @@ OBSLogViewer::~OBSLogViewer()
 			  saveGeometry().toBase64().constData());
 }
 
-void OBSLogViewer::ToggleShowStartup(bool checked)
+void OBSLogViewer::on_showStartup_clicked(bool checked)
 {
 	config_set_bool(App()->GlobalConfig(), "LogViewer", "ShowLogStartup",
 			checked);
@@ -106,13 +70,21 @@ void OBSLogViewer::InitLog()
 		in.setCodec("UTF-8");
 #endif
 
+		QTextDocument *doc = ui->textArea->document();
+		QTextCursor cursor(doc);
+		cursor.movePosition(QTextCursor::End);
+		cursor.beginEditBlock();
 		while (!in.atEnd()) {
 			QString line = in.readLine();
-			AddLine(LOG_INFO, line);
+			cursor.insertText(line);
+			cursor.insertBlock();
 		}
+		cursor.endEditBlock();
 
 		file.close();
 	}
+	QScrollBar *scroll = ui->textArea->verticalScrollBar();
+	scroll->setValue(scroll->maximum());
 
 	obsLogViewer = this;
 }
@@ -123,37 +95,35 @@ void OBSLogViewer::AddLine(int type, const QString &str)
 
 	switch (type) {
 	case LOG_WARNING:
-		msg = QStringLiteral("<font color=\"#c08000\">") + msg +
-		      QStringLiteral("</font>");
+		msg = QString("<font color=\"#c08000\">%1</font>").arg(msg);
 		break;
 	case LOG_ERROR:
-		msg = QStringLiteral("<font color=\"#c00000\">") + msg +
-		      QStringLiteral("</font>");
+		msg = QString("<font color=\"#c00000\">%1</font>").arg(msg);
+		break;
+	default:
+		msg = QString("<font>%1</font>").arg(msg);
 		break;
 	}
 
-	QScrollBar *scroll = textArea->verticalScrollBar();
+	QScrollBar *scroll = ui->textArea->verticalScrollBar();
 	bool bottomScrolled = scroll->value() >= scroll->maximum() - 10;
 
 	if (bottomScrolled)
 		scroll->setValue(scroll->maximum());
 
-	QTextCursor newCursor = textArea->textCursor();
-	newCursor.movePosition(QTextCursor::End);
-	newCursor.insertHtml(
-		QStringLiteral("<pre style=\"white-space: pre-wrap\">") + msg +
-		QStringLiteral("<br></pre>"));
+	QTextDocument *doc = ui->textArea->document();
+	QTextCursor cursor(doc);
+	cursor.movePosition(QTextCursor::End);
+	cursor.beginEditBlock();
+	cursor.insertHtml(msg);
+	cursor.insertBlock();
+	cursor.endEditBlock();
 
 	if (bottomScrolled)
 		scroll->setValue(scroll->maximum());
 }
 
-void OBSLogViewer::ClearText()
-{
-	textArea->clear();
-}
-
-void OBSLogViewer::OpenFile()
+void OBSLogViewer::on_openButton_clicked()
 {
 	char logDir[512];
 	if (GetConfigPath(logDir, sizeof(logDir), "obs-studio/logs") <= 0)
