@@ -60,6 +60,8 @@ enum obs_frontend_event {
 	OBS_FRONTEND_EVENT_SCRIPTING_SHUTDOWN,
 	OBS_FRONTEND_EVENT_PROFILE_RENAMED,
 	OBS_FRONTEND_EVENT_SCENE_COLLECTION_RENAMED,
+	OBS_FRONTEND_EVENT_THEME_CHANGED,
+	OBS_FRONTEND_EVENT_SCREENSHOT_TAKEN,
 };
 
 /* ------------------------------------------------------------------------- */
@@ -228,6 +230,12 @@ EXPORT void obs_frontend_open_source_interaction(obs_source_t *source);
 EXPORT char *obs_frontend_get_current_record_output_path(void);
 EXPORT const char *obs_frontend_get_locale_string(const char *string);
 
+EXPORT bool obs_frontend_is_theme_dark(void);
+
+EXPORT char *obs_frontend_get_last_recording(void);
+EXPORT char *obs_frontend_get_last_screenshot(void);
+EXPORT char *obs_frontend_get_last_replay(void);
+
 /* ------------------------------------------------------------------------- */
 
 #ifdef __cplusplus
