@@ -39,6 +39,7 @@ class QLayout;
 class QString;
 struct gs_window;
 class QLabel;
+class QToolBar;
 
 class OBSMessageBox {
 public:
@@ -122,3 +123,5 @@ QStringList OpenFiles(QWidget *parent, QString title, QString path,
 
 void TruncateLabel(QLabel *label, QString newText,
 		   int length = MAX_LABEL_LENGTH);
+
+void RefreshToolBarStyling(QToolBar *toolBar);
