@@ -47,6 +47,9 @@ public:
 
 	void paint(QPainter *painter, const QStyleOptionViewItem &option,
 		   const QModelIndex &index) const override;
+
+protected:
+	bool eventFilter(QObject *object, QEvent *event) override;
 };
 
 void SetupVisibilityItem(QListWidget *list, QListWidgetItem *item,
