@@ -24,6 +24,7 @@
 #include <vector>
 #include <string>
 #include <mutex>
+#include <unordered_set>
 
 using namespace std;
 using namespace json11;
@@ -246,6 +247,7 @@ enum state_t {
 	STATE_PENDING_DOWNLOAD,
 	STATE_DOWNLOADING,
 	STATE_DOWNLOADED,
+	STATE_ALREADY_DOWNLOADED,
 	STATE_INSTALL_FAILED,
 	STATE_INSTALLED,
 };
@@ -376,7 +378,7 @@ bool DownloadWorkerThread()
 
 	const DWORD enableHTTP2Flag = WINHTTP_PROTOCOL_FLAG_HTTP2;
 
-	HttpHandle hSession = WinHttpOpen(L"OBS Studio Updater/2.1",
+	HttpHandle hSession = WinHttpOpen(L"OBS Studio Updater/2.2",
 					  WINHTTP_ACCESS_TYPE_DEFAULT_PROXY,
 					  WINHTTP_NO_PROXY_NAME,
 					  WINHTTP_NO_PROXY_BYPASS, 0);
@@ -652,7 +654,8 @@ static inline bool is_64bit_file(const char *file)
 #define UPDATE_URL L"https://cdn-fastly.obsproject.com/update_studio"
 
 static bool AddPackageUpdateFiles(const Json &root, size_t idx,
-				  const wchar_t *tempPath)
+				  const wchar_t *tempPath,
+				  const wchar_t *branch)
 {
 	const Json &package = root[idx];
 	const Json &name = package["name"];
@@ -726,8 +729,9 @@ static bool AddPackageUpdateFiles(const Json &root, size_t idx,
 			return false;
 		}
 
-		StringCbPrintf(sourceURL, sizeof(sourceURL), L"%s/%s/%s",
-			       UPDATE_URL, wPackageName, updateFileName);
+		StringCbPrintf(sourceURL, sizeof(sourceURL), L"%s/%s/%s/%s",
+			       UPDATE_URL, branch, wPackageName,
+			       updateFileName);
 		StringCbPrintf(tempFilePath, sizeof(tempFilePath), L"%s\\%s",
 			       tempPath, updateHashStr);
 
@@ -1092,7 +1096,7 @@ static bool UpdateVS2019Redists(const Json &root)
 
 	const DWORD tlsProtocols = WINHTTP_FLAG_SECURE_PROTOCOL_TLS1_2;
 
-	HttpHandle hSession = WinHttpOpen(L"OBS Studio Updater/2.1",
+	HttpHandle hSession = WinHttpOpen(L"OBS Studio Updater/2.2",
 					  WINHTTP_ACCESS_TYPE_DEFAULT_PROXY,
 					  WINHTTP_NO_PROXY_NAME,
 					  WINHTTP_NO_PROXY_BYPASS, 0);
@@ -1284,6 +1288,8 @@ static bool Update(wchar_t *cmdLine)
 	 * Check if updating portable build      */
 
 	bool bIsPortable = false;
+	wstring branch = L"stable";
+	wstring appdata;
 
 	if (cmdLine[0]) {
 		int argc;
@@ -1292,10 +1298,20 @@ static bool Update(wchar_t *cmdLine)
 		if (argv) {
 			for (int i = 0; i < argc; i++) {
 				if (wcscmp(argv[i], L"Portable") == 0) {
+					// Legacy OBS
+					bIsPortable = true;
+					break;
+				} else if (wcsncmp(argv[i], L"--branch=", 9) ==
+					   0) {
+					branch = argv[i] + 9;
+				} else if (wcsncmp(argv[i], L"--appdata=",
+						   10) == 0) {
+					appdata = argv[i] + 10;
+				} else if (wcscmp(argv[i], L"--portable") ==
+					   0) {
 					bIsPortable = true;
 				}
 			}
-
 			LocalFree((HLOCAL)argv);
 		}
 	}
@@ -1310,18 +1326,16 @@ static bool Update(wchar_t *cmdLine)
 		GetCurrentDirectory(_countof(lpAppDataPath), lpAppDataPath);
 		StringCbCat(lpAppDataPath, sizeof(lpAppDataPath), L"\\config");
 	} else {
-		DWORD ret;
-		ret = GetEnvironmentVariable(L"OBS_USER_APPDATA_PATH",
-					     lpAppDataPath,
-					     _countof(lpAppDataPath));
-
-		if (ret >= _countof(lpAppDataPath)) {
-			Status(L"Update failed: Could not determine AppData "
-			       L"location");
-			return false;
-		}
-
-		if (!ret) {
+		if (!appdata.empty()) {
+			HRESULT hr = StringCbCopy(lpAppDataPath,
+						  sizeof(lpAppDataPath),
+						  appdata.c_str());
+			if (hr != S_OK) {
+				Status(L"Update failed: Could not determine AppData "
+				       L"location");
+				return false;
+			}
+		} else {
 			CoTaskMemPtr<wchar_t> pOut;
 			HRESULT hr = SHGetKnownFolderPath(
 				FOLDERID_RoamingAppData, KF_FLAG_DEFAULT,
@@ -1396,7 +1410,8 @@ static bool Update(wchar_t *cmdLine)
 
 	const Json::array &packages = root["packages"].array_items();
 	for (size_t i = 0; i < packages.size(); i++) {
-		if (!AddPackageUpdateFiles(packages, i, tempPath)) {
+		if (!AddPackageUpdateFiles(packages, i, tempPath,
+					   branch.c_str())) {
 			Status(L"Update failed: Failed to process update packages");
 			return false;
 		}
@@ -1481,7 +1496,11 @@ static bool Update(wchar_t *cmdLine)
 			  Z_BEST_COMPRESSION);
 		compressedJson.resize(compressSize);
 
-		bool success = !!HTTPPostData(PATCH_MANIFEST_URL,
+		wstring manifestUrl(PATCH_MANIFEST_URL);
+		if (branch != L"stable")
+			manifestUrl += L"?branch=" + branch;
+
+		bool success = !!HTTPPostData(manifestUrl.c_str(),
 					      (BYTE *)&compressedJson[0],
 					      (int)compressedJson.size(),
 					      L"Accept-Encoding: gzip",
@@ -1549,6 +1568,24 @@ static bool Update(wchar_t *cmdLine)
 					   source.c_str(), size);
 	}
 
+	/* ------------------------------------- *
+	 * Deduplicate Downloads                 */
+
+	unordered_set<wstring> tempFiles;
+	for (update_t &update : updates) {
+		if (update.patchable)
+			continue;
+
+		if (tempFiles.count(update.tempPath)) {
+			update.state = STATE_ALREADY_DOWNLOADED;
+			totalFileSize -= update.fileSize;
+			completedUpdates++;
+			continue;
+		}
+
+		tempFiles.insert(update.tempPath);
+	}
+
 	/* ------------------------------------- *
 	 * Download Updates                      */
 
@@ -1613,6 +1650,7 @@ static bool Update(wchar_t *cmdLine)
 	};
 
 	if (!bIsPortable) {
+		Status(L"Installing Virtual Camera...");
 		wchar_t regsvr[MAX_PATH];
 		wchar_t src[MAX_PATH];
 		wchar_t tmp[MAX_PATH];
@@ -1646,11 +1684,13 @@ static bool Update(wchar_t *cmdLine)
 	/* ------------------------------------- *
 	 * Update hook files and vulkan registry */
 
+	Status(L"Updating Game Capture hooks...");
 	UpdateHookFiles();
 
 	/* ------------------------------------- *
 	 * Finish                                */
 
+	Status(L"Cleaning up...");
 	/* If we get here, all updates installed successfully so we can purge
 	 * the old versions */
 	for (update_t &update : updates) {
@@ -1719,7 +1759,7 @@ static void CancelUpdate(bool quit)
 	}
 }
 
-static void LaunchOBS()
+static void LaunchOBS(bool portable)
 {
 	wchar_t cwd[MAX_PATH];
 	wchar_t newCwd[MAX_PATH];
@@ -1759,6 +1799,9 @@ static void LaunchOBS()
 	execInfo.lpDirectory = newCwd;
 	execInfo.nShow = SW_SHOWNORMAL;
 
+	if (portable)
+		execInfo.lpParameters = L"--portable";
+
 	ShellExecuteEx(&execInfo);
 }
 
@@ -1807,13 +1850,26 @@ static int RestartAsAdmin(LPCWSTR lpCmdLine, LPCWSTR cwd)
 		return 0;
 	}
 
+	/* If the admin is a different user, add the path to the user's
+	 * AppData to the command line so we can load the correct manifest. */
+	wstring elevatedCmdLine(lpCmdLine);
+	CoTaskMemPtr<wchar_t> pOut;
+	HRESULT hr = SHGetKnownFolderPath(FOLDERID_RoamingAppData,
+					  KF_FLAG_DEFAULT, nullptr, &pOut);
+	if (hr == S_OK) {
+		elevatedCmdLine += L" \"--appdata=";
+		elevatedCmdLine += pOut;
+		elevatedCmdLine += L"\"";
+	}
+
 	SHELLEXECUTEINFO shExInfo = {0};
 	shExInfo.cbSize = sizeof(shExInfo);
 	shExInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
 	shExInfo.hwnd = 0;
-	shExInfo.lpVerb = L"runas";        /* Operation to perform */
-	shExInfo.lpFile = myPath;          /* Application to start */
-	shExInfo.lpParameters = lpCmdLine; /* Additional parameters */
+	shExInfo.lpVerb = L"runas"; /* Operation to perform */
+	shExInfo.lpFile = myPath;   /* Application to start */
+	shExInfo.lpParameters =
+		elevatedCmdLine.c_str(); /* Additional parameters */
 	shExInfo.lpDirectory = cwd;
 	shExInfo.nShow = SW_NORMAL;
 	shExInfo.hInstApp = 0;
@@ -1822,14 +1878,6 @@ static int RestartAsAdmin(LPCWSTR lpCmdLine, LPCWSTR cwd)
 	 * windows :( */
 	AllowSetForegroundWindow(ASFW_ANY);
 
-	/* if the admin is a different user, save the path to the user's
-	 * appdata so we can load the correct manifest */
-	CoTaskMemPtr<wchar_t> pOut;
-	HRESULT hr = SHGetKnownFolderPath(FOLDERID_RoamingAppData,
-					  KF_FLAG_DEFAULT, nullptr, &pOut);
-	if (hr == S_OK)
-		SetEnvironmentVariable(L"OBS_USER_APPDATA_PATH", pOut);
-
 	if (ShellExecuteEx(&shExInfo)) {
 		DWORD exitCode;
 
@@ -1872,6 +1920,8 @@ int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR lpCmdLine, int)
 	GetCurrentDirectoryW(_countof(cwd) - 1, cwd);
 
 	is32bit = wcsstr(cwd, L"bin\\32bit") != nullptr;
+	bool isPortable = wcsstr(lpCmdLine, L"Portable") != nullptr ||
+			  wcsstr(lpCmdLine, L"--portable") != nullptr;
 
 	if (!IsWindows10OrGreater()) {
 		MessageBox(
@@ -1907,7 +1957,7 @@ int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR lpCmdLine, int)
 					nullptr);
 			SetCurrentDirectory(newPath);
 
-			LaunchOBS();
+			LaunchOBS(isPortable);
 		}
 
 		if (hLowMutex) {
@@ -1955,7 +2005,7 @@ int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR lpCmdLine, int)
 		WinHandle hMutex = OpenMutex(
 			SYNCHRONIZE, false, L"OBSUpdaterRunningAsNonAdminUser");
 		if (msg.wParam == 1 && !hMutex) {
-			LaunchOBS();
+			LaunchOBS(isPortable);
 		}
 
 		return (int)msg.wParam;
