@@ -24,6 +24,7 @@
 #define ZLIB_CONST
 
 #include <windows.h>
+#include <versionhelpers.h>
 #include <winhttp.h>
 #include <commctrl.h>
 #include <Wincrypt.h>
@@ -74,7 +75,7 @@
 #endif
 
 #include <util/windows/WinHandle.hpp>
-#include <jansson.h>
+#include <json11.hpp>
 #include "resource.h"
 
 bool HTTPGetFile(HINTERNET hConnect, const wchar_t *url,
