@@ -120,6 +120,7 @@ private:
 	int pageIndex = 0;
 	bool loading = true;
 	bool forceAuthReload = false;
+	bool forceUpdateCheck = false;
 	std::string savedTheme;
 	int sampleRateIndex = 0;
 	int channelIndex = 0;
@@ -146,6 +147,7 @@ private:
 	QString curQSVPreset;
 	QString curNVENCPreset;
 	QString curAMDPreset;
+	QString curAMDAV1Preset;
 
 	QString curAdvStreamEncoder;
 	QString curAdvRecordEncoder;
@@ -218,12 +220,6 @@ private:
 		EnableApplyButton(false);
 	}
 
-#ifdef _WIN32
-	bool aeroWasDisabled = false;
-	QCheckBox *toggleAero = nullptr;
-	void ToggleDisableAero(bool checked);
-#endif
-
 	void HookWidget(QWidget *widget, const char *signal, const char *slot);
 
 	bool QueryChanges();
@@ -255,6 +251,7 @@ private:
 	/* general */
 	void LoadLanguageList();
 	void LoadThemeList();
+	void LoadBranchesList();
 
 	/* stream */
 	void InitStreamPage();
