@@ -49,7 +49,7 @@ static QString MakeMissedFramesText(uint32_t total_lagged,
 }
 
 OBSBasicStats::OBSBasicStats(QWidget *parent, bool closeable)
-	: QWidget(parent),
+	: QFrame(parent),
 	  cpu_info(os_cpu_usage_info_start()),
 	  timer(this),
 	  recTimeLeft(this)
@@ -247,13 +247,6 @@ void OBSBasicStats::AddOutputLabels(QString name)
 	ol.megabytesSent = new QLabel(this);
 	ol.bitrate = new QLabel(this);
 
-	int newPointSize = ol.status->font().pointSize();
-	newPointSize *= 13;
-	newPointSize /= 10;
-	QString qss =
-		QString("font-size: %1pt").arg(QString::number(newPointSize));
-	ol.status->setStyleSheet(qss);
-
 	int col = 0;
 	int row = outputLabels.size() + 1;
 	outputLayout->addWidget(ol.name, row, col++);
@@ -459,9 +452,15 @@ void OBSBasicStats::ResetRecTimeLeft()
 
 void OBSBasicStats::RecordingTimeLeft()
 {
+	if (bitrates.empty())
+		return;
+
 	long double averageBitrate =
 		accumulate(bitrates.begin(), bitrates.end(), 0.0) /
 		(long double)bitrates.size();
+	if (averageBitrate == 0)
+		return;
+
 	long double bytesPerSec = (averageBitrate / 8.0l) * 1000.0l;
 	long double secondsUntilFull = (long double)num_bytes / bytesPerSec;
 
