@@ -127,11 +127,6 @@ static bool do_http_request(struct update_info *info, const char *url,
 		curl_easy_setopt(info->curl, CURLOPT_HEADERDATA, info);
 	}
 
-#if LIBCURL_VERSION_NUM >= 0x072400
-	// A lot of servers don't yet support ALPN
-	curl_easy_setopt(info->curl, CURLOPT_SSL_ENABLE_ALPN, 0);
-#endif
-
 	code = curl_easy_perform(info->curl);
 	if (code != CURLE_OK) {
 		warn("Remote update of URL \"%s\" failed: %s", url,
