@@ -641,11 +641,7 @@ static int obs_lua_register_source(lua_State *script)
 	if (!existing) {
 		ls.data = current_lua_script;
 
-		pthread_mutexattr_t mutexattr;
-		pthread_mutexattr_init(&mutexattr);
-		pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
-		pthread_mutex_init(&ls.definition_mutex, &mutexattr);
-		pthread_mutexattr_destroy(&mutexattr);
+		pthread_mutex_init_recursive(&ls.definition_mutex);
 
 		info.type_data = bmemdup(&ls, sizeof(ls));
 		info.free_type_data = obs_lua_source_free_type_data;
