@@ -66,7 +66,8 @@ gs_obj::~gs_obj()
 		next->prev_next = prev_next;
 }
 
-static bool screen_supports_hdr(gs_device_t *device, HMONITOR hMonitor)
+static gs_monitor_color_info get_monitor_color_info(gs_device_t *device,
+						    HMONITOR hMonitor)
 {
 	IDXGIFactory1 *factory1 = device->factory;
 	if (!factory1->IsCurrent()) {
@@ -75,7 +76,8 @@ static bool screen_supports_hdr(gs_device_t *device, HMONITOR hMonitor)
 		device->monitor_to_hdr.clear();
 	}
 
-	for (const std::pair<HMONITOR, bool> &pair : device->monitor_to_hdr) {
+	for (const std::pair<HMONITOR, gs_monitor_color_info> &pair :
+	     device->monitor_to_hdr) {
 		if (pair.first == hMonitor)
 			return pair.second;
 	}
@@ -96,15 +98,19 @@ static bool screen_supports_hdr(gs_device_t *device, HMONITOR hMonitor)
 					const bool hdr =
 						desc1.ColorSpace ==
 						DXGI_COLOR_SPACE_RGB_FULL_G2084_NONE_P2020;
-					device->monitor_to_hdr.emplace_back(
-						hMonitor, hdr);
-					return hdr;
+					return device->monitor_to_hdr
+						.emplace_back(
+							hMonitor,
+							gs_monitor_color_info(
+								hdr,
+								desc1.BitsPerColor))
+						.second;
 				}
 			}
 		}
 	}
 
-	return false;
+	return gs_monitor_color_info(false, 8);
 }
 
 static enum gs_color_space get_next_space(gs_device_t *device, HWND hwnd,
@@ -115,8 +121,12 @@ static enum gs_color_space get_next_space(gs_device_t *device, HWND hwnd,
 		const HMONITOR hMonitor =
 			MonitorFromWindow(hwnd, MONITOR_DEFAULTTONEAREST);
 		if (hMonitor) {
-			if (screen_supports_hdr(device, hMonitor))
+			const gs_monitor_color_info info =
+				get_monitor_color_info(device, hMonitor);
+			if (info.hdr)
 				next_space = GS_CS_709_SCRGB;
+			else if (info.bits_per_color > 8)
+				next_space = GS_CS_SRGB_16F;
 		}
 	}
 
@@ -126,7 +136,14 @@ static enum gs_color_space get_next_space(gs_device_t *device, HWND hwnd,
 static enum gs_color_format
 get_swap_format_from_space(gs_color_space space, gs_color_format sdr_format)
 {
-	return (space == GS_CS_709_SCRGB) ? GS_RGBA16F : sdr_format;
+	gs_color_format format = sdr_format;
+	switch (space) {
+	case GS_CS_SRGB_16F:
+	case GS_CS_709_SCRGB:
+		format = GS_RGBA16F;
+	}
+
+	return format;
 }
 
 static inline enum gs_color_space
@@ -313,7 +330,8 @@ void gs_device::InitCompiler()
 	int ver = 49;
 
 	while (ver > 30) {
-		sprintf(d3dcompiler, "D3DCompiler_%02d.dll", ver);
+		snprintf(d3dcompiler, sizeof(d3dcompiler),
+			 "D3DCompiler_%02d.dll", ver);
 
 		HMODULE module = LoadLibraryA(d3dcompiler);
 		if (module) {
@@ -346,45 +364,6 @@ void gs_device::InitFactory()
 		throw UnsupportedHWError("Failed to create DXGIFactory", hr);
 }
 
-#define VENDOR_ID_INTEL 0x8086
-#define IGPU_MEM (512 * 1024 * 1024)
-
-void gs_device::ReorderAdapters(uint32_t &adapterIdx)
-{
-	std::vector<uint32_t> adapterOrder;
-	ComPtr<IDXGIAdapter> adapter;
-	DXGI_ADAPTER_DESC desc;
-	uint32_t iGPUIndex = 0;
-	bool hasIGPU = false;
-	bool hasDGPU = false;
-	int idx = 0;
-
-	while (SUCCEEDED(factory->EnumAdapters(idx, &adapter))) {
-		if (SUCCEEDED(adapter->GetDesc(&desc))) {
-			if (desc.VendorId == VENDOR_ID_INTEL) {
-				if (desc.DedicatedVideoMemory <= IGPU_MEM) {
-					hasIGPU = true;
-					iGPUIndex = (uint32_t)idx;
-				} else {
-					hasDGPU = true;
-				}
-			}
-		}
-
-		adapterOrder.push_back((uint32_t)idx++);
-	}
-
-	/* Intel specific adapter check for Intel integrated and Intel
-	 * dedicated. If both exist, then change adapter priority so that the
-	 * integrated comes first for the sake of improving overall
-	 * performance */
-	if (hasIGPU && hasDGPU) {
-		adapterOrder.erase(adapterOrder.begin() + iGPUIndex);
-		adapterOrder.insert(adapterOrder.begin(), iGPUIndex);
-		adapterIdx = adapterOrder[adapterIdx];
-	}
-}
-
 void gs_device::InitAdapter(uint32_t adapterIdx)
 {
 	HRESULT hr = factory->EnumAdapters1(adapterIdx, &adapter);
@@ -923,7 +902,6 @@ gs_device::gs_device(uint32_t adapterIdx)
 
 	InitCompiler();
 	InitFactory();
-	ReorderAdapters(adapterIdx);
 	InitAdapter(adapterIdx);
 	InitDevice(adapterIdx);
 	device_set_render_target(this, NULL, NULL);
@@ -1246,6 +1224,7 @@ static inline void LogAdapterMonitors(IDXGIAdapter1 *adapter)
 			target.monitorFriendlyDeviceName[0] = 0;
 		}
 
+		UINT bits_per_color = 8;
 		DXGI_COLOR_SPACE_TYPE type =
 			DXGI_COLOR_SPACE_RGB_FULL_G22_NONE_P709;
 		FLOAT min_luminance = 0.f;
@@ -1253,6 +1232,7 @@ static inline void LogAdapterMonitors(IDXGIAdapter1 *adapter)
 		FLOAT max_full_frame_luminance = 0.f;
 		DXGI_OUTPUT_DESC1 desc1;
 		if (GetOutputDesc1(output, &desc1)) {
+			bits_per_color = desc1.BitsPerColor;
 			type = desc1.ColorSpace;
 			min_luminance = desc1.MinLuminance;
 			max_luminance = desc1.MaxLuminance;
@@ -1282,13 +1262,14 @@ static inline void LogAdapterMonitors(IDXGIAdapter1 *adapter)
 		     "\t    size={%d, %d}\n"
 		     "\t    attached=%s\n"
 		     "\t    refresh=%u\n"
+		     "\t    bits_per_color=%u\n"
 		     "\t    space=%s\n"
 		     "\t    sdr_white_nits=%lu\n"
 		     "\t    nit_range=[min=%f, max=%f, max_full_frame=%f]",
 		     i, target.monitorFriendlyDeviceName, rect.left, rect.top,
 		     rect.right - rect.left, rect.bottom - rect.top,
-		     desc.AttachedToDesktop ? "true" : "false", refresh, space,
-		     nits, min_luminance, max_luminance,
+		     desc.AttachedToDesktop ? "true" : "false", refresh,
+		     bits_per_color, space, nits, min_luminance, max_luminance,
 		     max_full_frame_luminance);
 	}
 }
@@ -3068,7 +3049,7 @@ extern "C" EXPORT bool device_p010_available(gs_device_t *device)
 extern "C" EXPORT bool device_is_monitor_hdr(gs_device_t *device, void *monitor)
 {
 	const HMONITOR hMonitor = static_cast<HMONITOR>(monitor);
-	return screen_supports_hdr(device, hMonitor);
+	return get_monitor_color_info(device, hMonitor).hdr;
 }
 
 extern "C" EXPORT void device_debug_marker_begin(gs_device_t *,
