@@ -103,7 +103,9 @@ void gs_texture_2d::InitTexture(const uint8_t *const *data)
 	td.Height = height;
 	td.MipLevels = genMipmaps ? 0 : levels;
 	td.ArraySize = type == GS_TEXTURE_CUBE ? 6 : 1;
-	td.Format = nv12 ? DXGI_FORMAT_NV12 : dxgiFormatResource;
+	td.Format = twoPlane ? ((format == GS_R16) ? DXGI_FORMAT_P010
+						   : DXGI_FORMAT_NV12)
+			     : dxgiFormatResource;
 	td.BindFlags = D3D11_BIND_SHADER_RESOURCE;
 	td.SampleDesc.Count = 1;
 	td.CPUAccessFlags = isDynamic ? D3D11_CPU_ACCESS_WRITE : 0;
@@ -266,7 +268,7 @@ gs_texture_2d::gs_texture_2d(gs_device_t *device, uint32_t width,
 			     uint32_t height, gs_color_format colorFormat,
 			     uint32_t levels, const uint8_t *const *data,
 			     uint32_t flags_, gs_texture_type type,
-			     bool gdiCompatible, bool nv12_)
+			     bool gdiCompatible, bool twoPlane_)
 	: gs_texture(device, gs_type::gs_texture_2d, type, levels, colorFormat),
 	  width(width),
 	  height(height),
@@ -280,7 +282,7 @@ gs_texture_2d::gs_texture_2d(gs_device_t *device, uint32_t width,
 	  isShared((flags_ & SHARED_FLAGS) != 0),
 	  genMipmaps((flags_ & GS_BUILD_MIPMAPS) != 0),
 	  sharedHandle(GS_INVALID_HANDLE),
-	  nv12(nv12_)
+	  twoPlane(twoPlane_)
 {
 	InitTexture(data);
 	InitResourceView();
@@ -296,22 +298,26 @@ gs_texture_2d::gs_texture_2d(gs_device_t *device, ID3D11Texture2D *nv12tex,
 	  isDynamic((flags_ & GS_DYNAMIC) != 0),
 	  isShared((flags_ & SHARED_FLAGS) != 0),
 	  genMipmaps((flags_ & GS_BUILD_MIPMAPS) != 0),
-	  nv12(true),
+	  twoPlane(true),
 	  texture(nv12tex)
 {
 	texture->GetDesc(&td);
 
+	const bool p010 = td.Format == DXGI_FORMAT_P010;
+	const DXGI_FORMAT dxgi_format = p010 ? DXGI_FORMAT_R16G16_UNORM
+					     : DXGI_FORMAT_R8G8_UNORM;
+
 	this->type = GS_TEXTURE_2D;
-	this->format = GS_R8G8;
+	this->format = p010 ? GS_RG16 : GS_R8G8;
 	this->flags = flags_;
 	this->levels = 1;
 	this->device = device;
 	this->chroma = true;
 	this->width = td.Width / 2;
 	this->height = td.Height / 2;
-	this->dxgiFormatResource = DXGI_FORMAT_R8G8_UNORM;
-	this->dxgiFormatView = DXGI_FORMAT_R8G8_UNORM;
-	this->dxgiFormatViewLinear = DXGI_FORMAT_R8G8_UNORM;
+	this->dxgiFormatResource = dxgi_format;
+	this->dxgiFormatView = dxgi_format;
+	this->dxgiFormatViewLinear = dxgi_format;
 
 	InitResourceView();
 	if (isRenderTarget)
