@@ -12,7 +12,8 @@ extern "C" {
 EXPORT BOOL winrt_capture_supported();
 EXPORT BOOL winrt_capture_cursor_toggle_supported();
 EXPORT struct winrt_capture *winrt_capture_init_window(BOOL cursor, HWND window,
-						       BOOL client_area);
+						       BOOL client_area,
+						       BOOL force_sdr);
 EXPORT struct winrt_capture *winrt_capture_init_monitor(BOOL cursor,
 							HMONITOR monitor);
 EXPORT void winrt_capture_free(struct winrt_capture *capture);
