@@ -1017,12 +1017,14 @@ static inline int ep_parse_param_assign_intfloat(struct effect_parser *ep,
 		float f = (float)os_strtod(ep->cfp.cur_token->str.array);
 		if (is_negative)
 			f = -f;
-		da_push_back_array(param->default_val, &f, sizeof(float));
+		da_push_back_array(param->default_val, (uint8_t *)&f,
+				   sizeof(float));
 	} else {
 		long l = strtol(ep->cfp.cur_token->str.array, NULL, 10);
 		if (is_negative)
 			l = -l;
-		da_push_back_array(param->default_val, &l, sizeof(long));
+		da_push_back_array(param->default_val, (uint8_t *)&l,
+				   sizeof(long));
 	}
 
 	return PARSE_SUCCESS;
@@ -1036,11 +1038,13 @@ static inline int ep_parse_param_assign_bool(struct effect_parser *ep,
 
 	if (cf_token_is(&ep->cfp, "true")) {
 		long l = 1;
-		da_push_back_array(param->default_val, &l, sizeof(long));
+		da_push_back_array(param->default_val, (uint8_t *)&l,
+				   sizeof(long));
 		return PARSE_SUCCESS;
 	} else if (cf_token_is(&ep->cfp, "false")) {
 		long l = 0;
-		da_push_back_array(param->default_val, &l, sizeof(long));
+		da_push_back_array(param->default_val, (uint8_t *)&l,
+				   sizeof(long));
 		return PARSE_SUCCESS;
 	}
 
