@@ -112,6 +112,17 @@ void video_frame_init(struct video_frame *frame, enum video_format format,
 		frame->linesize[2] = width;
 		break;
 
+	case VIDEO_FORMAT_I412:
+		size = width * height * 2;
+		ALIGN_SIZE(size, alignment);
+		frame->data[0] = bmalloc(size * 3);
+		frame->data[1] = (uint8_t *)frame->data[0] + size;
+		frame->data[2] = (uint8_t *)frame->data[1] + size;
+		frame->linesize[0] = width * 2;
+		frame->linesize[1] = width * 2;
+		frame->linesize[2] = width * 2;
+		break;
+
 	case VIDEO_FORMAT_BGR3:
 		size = width * height * 3;
 		ALIGN_SIZE(size, alignment);
@@ -139,6 +150,27 @@ void video_frame_init(struct video_frame *frame, enum video_format format,
 		break;
 	}
 
+	case VIDEO_FORMAT_I210: {
+		size = width * height * 2;
+		ALIGN_SIZE(size, alignment);
+		offsets[0] = size;
+		const uint32_t half_width = (width + 1) / 2;
+		const uint32_t half_area = half_width * height;
+		const uint32_t half_area_size = 2 * half_area;
+		size += half_area_size;
+		ALIGN_SIZE(size, alignment);
+		offsets[1] = size;
+		size += half_area_size;
+		ALIGN_SIZE(size, alignment);
+		frame->data[0] = bmalloc(size);
+		frame->data[1] = (uint8_t *)frame->data[0] + offsets[0];
+		frame->data[2] = (uint8_t *)frame->data[0] + offsets[1];
+		frame->linesize[0] = width * 2;
+		frame->linesize[1] = half_width * 2;
+		frame->linesize[2] = half_width * 2;
+		break;
+	}
+
 	case VIDEO_FORMAT_I40A: {
 		size = width * height;
 		ALIGN_SIZE(size, alignment);
@@ -211,6 +243,66 @@ void video_frame_init(struct video_frame *frame, enum video_format format,
 		frame->linesize[2] = width;
 		frame->linesize[3] = width;
 		break;
+
+	case VIDEO_FORMAT_YA2L: {
+		const uint32_t linesize = width * 2;
+		const uint32_t plane_size = linesize * height;
+		size = plane_size;
+		ALIGN_SIZE(size, alignment);
+		offsets[0] = size;
+		size += plane_size;
+		ALIGN_SIZE(size, alignment);
+		offsets[1] = size;
+		size += plane_size;
+		ALIGN_SIZE(size, alignment);
+		offsets[2] = size;
+		size += plane_size;
+		ALIGN_SIZE(size, alignment);
+		frame->data[0] = bmalloc(size);
+		frame->data[1] = (uint8_t *)frame->data[0] + offsets[0];
+		frame->data[2] = (uint8_t *)frame->data[0] + offsets[1];
+		frame->data[3] = (uint8_t *)frame->data[0] + offsets[2];
+		frame->linesize[0] = linesize;
+		frame->linesize[1] = linesize;
+		frame->linesize[2] = linesize;
+		frame->linesize[3] = linesize;
+		break;
+	}
+
+	case VIDEO_FORMAT_I010: {
+		size = width * height * 2;
+		ALIGN_SIZE(size, alignment);
+		offsets[0] = size;
+		const uint32_t half_width = (width + 1) / 2;
+		const uint32_t half_height = (height + 1) / 2;
+		const uint32_t quarter_area = half_width * half_height;
+		size += quarter_area * 2;
+		ALIGN_SIZE(size, alignment);
+		offsets[1] = size;
+		size += quarter_area * 2;
+		ALIGN_SIZE(size, alignment);
+		frame->data[0] = bmalloc(size);
+		frame->data[1] = (uint8_t *)frame->data[0] + offsets[0];
+		frame->data[2] = (uint8_t *)frame->data[0] + offsets[1];
+		frame->linesize[0] = width * 2;
+		frame->linesize[1] = half_width * 2;
+		frame->linesize[2] = half_width * 2;
+		break;
+	}
+
+	case VIDEO_FORMAT_P010: {
+		size = width * height * 2;
+		ALIGN_SIZE(size, alignment);
+		offsets[0] = size;
+		const uint32_t cbcr_width = (width + 1) & (UINT32_MAX - 1);
+		size += cbcr_width * ((height + 1) / 2) * 2;
+		ALIGN_SIZE(size, alignment);
+		frame->data[0] = bmalloc(size);
+		frame->data[1] = (uint8_t *)frame->data[0] + offsets[0];
+		frame->linesize[0] = width * 2;
+		frame->linesize[1] = cbcr_width * 2;
+		break;
+	}
 	}
 }
 
@@ -222,12 +314,14 @@ void video_frame_copy(struct video_frame *dst, const struct video_frame *src,
 		return;
 
 	case VIDEO_FORMAT_I420:
+	case VIDEO_FORMAT_I010:
 		memcpy(dst->data[0], src->data[0], src->linesize[0] * cy);
 		memcpy(dst->data[1], src->data[1], src->linesize[1] * cy / 2);
 		memcpy(dst->data[2], src->data[2], src->linesize[2] * cy / 2);
 		break;
 
 	case VIDEO_FORMAT_NV12:
+	case VIDEO_FORMAT_P010:
 		memcpy(dst->data[0], src->data[0], src->linesize[0] * cy);
 		memcpy(dst->data[1], src->data[1], src->linesize[1] * cy / 2);
 		break;
@@ -246,6 +340,8 @@ void video_frame_copy(struct video_frame *dst, const struct video_frame *src,
 
 	case VIDEO_FORMAT_I444:
 	case VIDEO_FORMAT_I422:
+	case VIDEO_FORMAT_I210:
+	case VIDEO_FORMAT_I412:
 		memcpy(dst->data[0], src->data[0], src->linesize[0] * cy);
 		memcpy(dst->data[1], src->data[1], src->linesize[1] * cy);
 		memcpy(dst->data[2], src->data[2], src->linesize[2] * cy);
@@ -260,6 +356,7 @@ void video_frame_copy(struct video_frame *dst, const struct video_frame *src,
 
 	case VIDEO_FORMAT_I42A:
 	case VIDEO_FORMAT_YUVA:
+	case VIDEO_FORMAT_YA2L:
 		memcpy(dst->data[0], src->data[0], src->linesize[0] * cy);
 		memcpy(dst->data[1], src->data[1], src->linesize[1] * cy);
 		memcpy(dst->data[2], src->data[2], src->linesize[2] * cy);
