@@ -15,17 +15,18 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
 
-#include "obs.h"
 #include "obs-avc.h"
+
+#include "obs.h"
+#include "obs-nal.h"
 #include "util/array-serializer.h"
 
 bool obs_avc_keyframe(const uint8_t *data, size_t size)
 {
 	const uint8_t *nal_start, *nal_end;
 	const uint8_t *end = data + size;
-	int type;
 
-	nal_start = obs_avc_find_startcode(data, end);
+	nal_start = obs_nal_find_startcode(data, end);
 	while (true) {
 		while (nal_start < end && !*(nal_start++))
 			;
@@ -33,80 +34,42 @@ bool obs_avc_keyframe(const uint8_t *data, size_t size)
 		if (nal_start == end)
 			break;
 
-		type = nal_start[0] & 0x1F;
+		const uint8_t type = nal_start[0] & 0x1F;
 
 		if (type == OBS_NAL_SLICE_IDR || type == OBS_NAL_SLICE)
-			return (type == OBS_NAL_SLICE_IDR);
+			return type == OBS_NAL_SLICE_IDR;
 
-		nal_end = obs_avc_find_startcode(nal_start, end);
+		nal_end = obs_nal_find_startcode(nal_start, end);
 		nal_start = nal_end;
 	}
 
 	return false;
 }
 
-/* NOTE: I noticed that FFmpeg does some unusual special handling of certain
- * scenarios that I was unaware of, so instead of just searching for {0, 0, 1}
- * we'll just use the code from FFmpeg - http://www.ffmpeg.org/ */
-static const uint8_t *ff_avc_find_startcode_internal(const uint8_t *p,
-						     const uint8_t *end)
-{
-	const uint8_t *a = p + 4 - ((intptr_t)p & 3);
-
-	for (end -= 3; p < a && p < end; p++) {
-		if (p[0] == 0 && p[1] == 0 && p[2] == 1)
-			return p;
-	}
-
-	for (end -= 3; p < end; p += 4) {
-		uint32_t x = *(const uint32_t *)p;
-
-		if ((x - 0x01010101) & (~x) & 0x80808080) {
-			if (p[1] == 0) {
-				if (p[0] == 0 && p[2] == 1)
-					return p;
-				if (p[2] == 0 && p[3] == 1)
-					return p + 1;
-			}
-
-			if (p[3] == 0) {
-				if (p[2] == 0 && p[4] == 1)
-					return p + 2;
-				if (p[4] == 0 && p[5] == 1)
-					return p + 3;
-			}
-		}
-	}
-
-	for (end += 3; p < end; p++) {
-		if (p[0] == 0 && p[1] == 0 && p[2] == 1)
-			return p;
-	}
-
-	return end + 3;
-}
-
 const uint8_t *obs_avc_find_startcode(const uint8_t *p, const uint8_t *end)
 {
-	const uint8_t *out = ff_avc_find_startcode_internal(p, end);
-	if (p < out && out < end && !out[-1])
-		out--;
-	return out;
+	return obs_nal_find_startcode(p, end);
 }
 
-static inline int get_drop_priority(int priority)
+static int compute_avc_keyframe_priority(const uint8_t *nal_start,
+					 bool *is_keyframe, int priority)
 {
+	const int type = nal_start[0] & 0x1F;
+	if (type == OBS_NAL_SLICE_IDR)
+		*is_keyframe = true;
+
+	const int new_priority = nal_start[0] >> 5;
+	if (priority < new_priority)
+		priority = new_priority;
+
 	return priority;
 }
 
 static void serialize_avc_data(struct serializer *s, const uint8_t *data,
 			       size_t size, bool *is_keyframe, int *priority)
 {
-	const uint8_t *nal_start, *nal_end;
-	const uint8_t *end = data + size;
-	int type;
-
-	nal_start = obs_avc_find_startcode(data, end);
+	const uint8_t *const end = data + size;
+	const uint8_t *nal_start = obs_nal_find_startcode(data, end);
 	while (true) {
 		while (nal_start < end && !*(nal_start++))
 			;
@@ -114,18 +77,14 @@ static void serialize_avc_data(struct serializer *s, const uint8_t *data,
 		if (nal_start == end)
 			break;
 
-		type = nal_start[0] & 0x1F;
+		*priority = compute_avc_keyframe_priority(
+			nal_start, is_keyframe, *priority);
 
-		if (type == OBS_NAL_SLICE_IDR || type == OBS_NAL_SLICE) {
-			if (is_keyframe)
-				*is_keyframe = (type == OBS_NAL_SLICE_IDR);
-			if (priority)
-				*priority = nal_start[0] >> 5;
-		}
-
-		nal_end = obs_avc_find_startcode(nal_start, end);
-		s_wb32(s, (uint32_t)(nal_end - nal_start));
-		s_write(s, nal_start, nal_end - nal_start);
+		const uint8_t *const nal_end =
+			obs_nal_find_startcode(nal_start, end);
+		const size_t nal_size = nal_end - nal_start;
+		s_wb32(s, (uint32_t)nal_size);
+		s_write(s, nal_start, nal_size);
 		nal_start = nal_end;
 	}
 }
@@ -146,7 +105,31 @@ void obs_parse_avc_packet(struct encoder_packet *avc_packet,
 
 	avc_packet->data = output.bytes.array + sizeof(ref);
 	avc_packet->size = output.bytes.num - sizeof(ref);
-	avc_packet->drop_priority = get_drop_priority(avc_packet->priority);
+	avc_packet->drop_priority = avc_packet->priority;
+}
+
+int obs_parse_avc_packet_priority(const struct encoder_packet *packet)
+{
+	int priority = packet->priority;
+
+	const uint8_t *const data = packet->data;
+	const uint8_t *const end = data + packet->size;
+	const uint8_t *nal_start = obs_nal_find_startcode(data, end);
+	while (true) {
+		while (nal_start < end && !*(nal_start++))
+			;
+
+		if (nal_start == end)
+			break;
+
+		bool unused;
+		priority = compute_avc_keyframe_priority(nal_start, &unused,
+							 priority);
+
+		nal_start = obs_nal_find_startcode(nal_start, end);
+	}
+
+	return priority;
 }
 
 static inline bool has_start_code(const uint8_t *data)
@@ -164,7 +147,7 @@ static void get_sps_pps(const uint8_t *data, size_t size, const uint8_t **sps,
 	const uint8_t *end = data + size;
 	int type;
 
-	nal_start = obs_avc_find_startcode(data, end);
+	nal_start = obs_nal_find_startcode(data, end);
 	while (true) {
 		while (nal_start < end && !*(nal_start++))
 			;
@@ -172,7 +155,7 @@ static void get_sps_pps(const uint8_t *data, size_t size, const uint8_t **sps,
 		if (nal_start == end)
 			break;
 
-		nal_end = obs_avc_find_startcode(nal_start, end);
+		nal_end = obs_nal_find_startcode(nal_start, end);
 
 		type = nal_start[0] & 0x1F;
 		if (type == OBS_NAL_SPS) {
@@ -233,13 +216,12 @@ void obs_extract_avc_headers(const uint8_t *packet, size_t size,
 	DARRAY(uint8_t) sei;
 	const uint8_t *nal_start, *nal_end, *nal_codestart;
 	const uint8_t *end = packet + size;
-	int type;
 
 	da_init(new_packet);
 	da_init(header);
 	da_init(sei);
 
-	nal_start = obs_avc_find_startcode(packet, end);
+	nal_start = obs_nal_find_startcode(packet, end);
 	nal_end = NULL;
 	while (nal_end != end) {
 		nal_codestart = nal_start;
@@ -250,9 +232,9 @@ void obs_extract_avc_headers(const uint8_t *packet, size_t size,
 		if (nal_start == end)
 			break;
 
-		type = nal_start[0] & 0x1F;
+		const uint8_t type = nal_start[0] & 0x1F;
 
-		nal_end = obs_avc_find_startcode(nal_start, end);
+		nal_end = obs_nal_find_startcode(nal_start, end);
 		if (!nal_end)
 			nal_end = end;
 
