@@ -250,40 +250,24 @@ void set_deinterlace_texture_size(obs_source_t *source)
 	}
 }
 
-static inline struct obs_source_frame *get_prev_frame(obs_source_t *source,
-						      bool *updated)
-{
-	struct obs_source_frame *frame = NULL;
-
-	pthread_mutex_lock(&source->async_mutex);
-
-	*updated = source->cur_async_frame != NULL;
-	frame = source->prev_async_frame;
-	source->prev_async_frame = NULL;
-
-	if (frame)
-		os_atomic_inc_long(&frame->refs);
-
-	pthread_mutex_unlock(&source->async_mutex);
-
-	return frame;
-}
-
 void deinterlace_update_async_video(obs_source_t *source)
 {
-	struct obs_source_frame *frame;
-	bool updated;
-
 	if (source->deinterlace_rendered)
 		return;
 
-	frame = get_prev_frame(source, &updated);
-
 	source->deinterlace_rendered = true;
-	if (frame)
-		frame = filter_async_video(source, frame);
+
+	pthread_mutex_lock(&source->async_mutex);
+
+	const bool updated = source->cur_async_frame != NULL;
+	struct obs_source_frame *frame = source->prev_async_frame;
+	source->prev_async_frame = NULL;
+
+	pthread_mutex_unlock(&source->async_mutex);
 
 	if (frame) {
+		os_atomic_inc_long(&frame->refs);
+
 		if (set_async_texture_size(source, frame)) {
 			update_async_textures(source, frame,
 					      source->async_prev_textures,
