@@ -27,6 +27,7 @@
 #include <iwscapi.h>
 
 static uint32_t win_ver = 0;
+static uint32_t win_build = 0;
 
 const char *get_module_extension(void)
 {
@@ -107,6 +108,13 @@ static void log_processor_cores(void)
 	     os_get_physical_cores(), os_get_logical_cores());
 }
 
+static void log_emulation_status(void)
+{
+	if (os_get_emulation_status()) {
+		blog(LOG_WARNING, "Windows ARM64: Running with x64 emulation");
+	}
+}
+
 static void log_available_memory(void)
 {
 	MEMORYSTATUSEX ms;
@@ -137,10 +145,13 @@ static void log_windows_version(void)
 	bool b64 = is_64_bit_windows();
 	const char *windows_bitness = b64 ? "64" : "32";
 
+	bool arm64 = is_arm64_windows();
+	const char *arm64_windows = arm64 ? "ARM " : "";
+
 	blog(LOG_INFO,
-	     "Windows Version: %d.%d Build %d (release: %s; revision: %d; %s-bit)",
+	     "Windows Version: %d.%d Build %d (release: %s; revision: %d; %s%s-bit)",
 	     ver.major, ver.minor, ver.build, release_id, ver.revis,
-	     windows_bitness);
+	     arm64_windows, windows_bitness);
 }
 
 static void log_admin_status(void)
@@ -163,36 +174,6 @@ static void log_admin_status(void)
 	     success ? "true" : "false");
 }
 
-typedef HRESULT(WINAPI *dwm_is_composition_enabled_t)(BOOL *);
-
-static void log_aero(void)
-{
-	dwm_is_composition_enabled_t composition_enabled = NULL;
-
-	const char *aeroMessage =
-		win_ver >= 0x602
-			? " (Aero is always on for windows 8 and above)"
-			: "";
-
-	HMODULE dwm = LoadLibraryW(L"dwmapi");
-	BOOL bComposition = true;
-
-	if (!dwm) {
-		return;
-	}
-
-	composition_enabled = (dwm_is_composition_enabled_t)GetProcAddress(
-		dwm, "DwmIsCompositionEnabled");
-	if (!composition_enabled) {
-		FreeLibrary(dwm);
-		return;
-	}
-
-	composition_enabled(&bComposition);
-	blog(LOG_INFO, "Aero is %s%s", bComposition ? "Enabled" : "Disabled",
-	     aeroMessage);
-}
-
 #define WIN10_GAME_BAR_REG_KEY \
 	L"Software\\Microsoft\\Windows\\CurrentVersion\\GameDVR"
 #define WIN10_GAME_DVR_POLICY_REG_KEY \
@@ -223,16 +204,16 @@ static void log_gaming_features(void)
 	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_BAR_REG_KEY,
 		      L"HistoricalCaptureEnabled", &game_dvr_bg_recording);
 	get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_MODE_REG_KEY,
-		      L"AllowAutoGameMode", &game_mode_enabled);
+		      L"AutoGameModeEnabled", &game_mode_enabled);
 	get_reg_dword(HKEY_LOCAL_MACHINE, WIN10_HAGS_REG_KEY, L"HwSchMode",
 		      &hags_enabled);
 
 	if (game_mode_enabled.status != ERROR_SUCCESS) {
 		get_reg_dword(HKEY_CURRENT_USER, WIN10_GAME_MODE_REG_KEY,
-			      L"AutoGameModeEnabled", &game_mode_enabled);
+			      L"AllowAutoGameMode", &game_mode_enabled);
 	}
 
-	blog(LOG_INFO, "Windows 10 Gaming Features:");
+	blog(LOG_INFO, "Windows 10/11 Gaming Features:");
 	if (game_bar_enabled.status == ERROR_SUCCESS) {
 		blog(LOG_INFO, "\tGame Bar: %s",
 		     (bool)game_bar_enabled.return_value ? "On" : "Off");
@@ -256,11 +237,18 @@ static void log_gaming_features(void)
 	if (game_mode_enabled.status == ERROR_SUCCESS) {
 		blog(LOG_INFO, "\tGame Mode: %s",
 		     (bool)game_mode_enabled.return_value ? "On" : "Off");
+	} else if (win_build >= 19042) {
+		// On by default in newer Windows 10 builds (no registry key set)
+		blog(LOG_INFO, "\tGame Mode: Probably On (no reg key set)");
 	}
 
 	if (hags_enabled.status == ERROR_SUCCESS) {
 		blog(LOG_INFO, "\tHardware GPU Scheduler: %s",
 		     (hags_enabled.return_value == 2) ? "On" : "Off");
+	} else if (win_build >= 22000) {
+		// On by default in Windows 11 (no registry key set)
+		blog(LOG_INFO,
+		     "\tHardware GPU Scheduler: Probably On (no reg key set)");
 	}
 }
 
@@ -393,13 +381,14 @@ void log_system_info(void)
 	get_win_ver(&ver);
 
 	win_ver = (ver.major << 8) | ver.minor;
+	win_build = ver.build;
 
 	log_processor_info();
 	log_processor_cores();
 	log_available_memory();
 	log_windows_version();
+	log_emulation_status();
 	log_admin_status();
-	log_aero();
 	log_gaming_features();
 	log_security_products();
 }
