@@ -20,12 +20,6 @@
 #include "c99defs.h"
 #include "base.h"
 
-#ifdef _DEBUG
-static int log_output_level = LOG_DEBUG;
-#else
-static int log_output_level = LOG_INFO;
-#endif
-
 static int crashing = 0;
 static void *log_param = NULL;
 static void *crash_param = NULL;
@@ -36,27 +30,25 @@ static void def_log_handler(int log_level, const char *format, va_list args,
 	char out[4096];
 	vsnprintf(out, sizeof(out), format, args);
 
-	if (log_level <= log_output_level) {
-		switch (log_level) {
-		case LOG_DEBUG:
-			fprintf(stdout, "debug: %s\n", out);
-			fflush(stdout);
-			break;
+	switch (log_level) {
+	case LOG_DEBUG:
+		fprintf(stdout, "debug: %s\n", out);
+		fflush(stdout);
+		break;
 
-		case LOG_INFO:
-			fprintf(stdout, "info: %s\n", out);
-			fflush(stdout);
-			break;
+	case LOG_INFO:
+		fprintf(stdout, "info: %s\n", out);
+		fflush(stdout);
+		break;
 
-		case LOG_WARNING:
-			fprintf(stdout, "warning: %s\n", out);
-			fflush(stdout);
-			break;
+	case LOG_WARNING:
+		fprintf(stdout, "warning: %s\n", out);
+		fflush(stdout);
+		break;
 
-		case LOG_ERROR:
-			fprintf(stderr, "error: %s\n", out);
-			fflush(stderr);
-		}
+	case LOG_ERROR:
+		fprintf(stderr, "error: %s\n", out);
+		fflush(stderr);
 	}
 
 	UNUSED_PARAMETER(param);
@@ -111,6 +103,7 @@ OBS_NORETURN void bcrash(const char *format, ...)
 	va_start(args, format);
 	crash_handler(format, args, crash_param);
 	va_end(args);
+	exit(0);
 }
 
 void blogva(int log_level, const char *format, va_list args)
