@@ -84,14 +84,16 @@ static inline void darray_reserve(const size_t element_size, struct darray *dst,
 				  const size_t capacity)
 {
 	void *ptr;
-	if (capacity == 0 || capacity <= dst->num)
+	if (capacity == 0 || capacity <= dst->capacity)
 		return;
 
 	ptr = bmalloc(element_size * capacity);
-	if (dst->num)
-		memcpy(ptr, dst->array, element_size * dst->num);
-	if (dst->array)
+	if (dst->array) {
+		if (dst->num)
+			memcpy(ptr, dst->array, element_size * dst->num);
+
 		bfree(dst->array);
+	}
 	dst->array = ptr;
 	dst->capacity = capacity;
 }
@@ -109,10 +111,12 @@ static inline void darray_ensure_capacity(const size_t element_size,
 	if (new_size > new_cap)
 		new_cap = new_size;
 	ptr = bmalloc(element_size * new_cap);
-	if (dst->capacity)
-		memcpy(ptr, dst->array, element_size * dst->capacity);
-	if (dst->array)
+	if (dst->array) {
+		if (dst->capacity)
+			memcpy(ptr, dst->array, element_size * dst->capacity);
+
 		bfree(dst->array);
+	}
 	dst->array = ptr;
 	dst->capacity = new_cap;
 }
@@ -405,6 +409,11 @@ static inline void darray_move_item(const size_t element_size,
 		return;
 
 	temp = malloc(element_size);
+	if (!temp) {
+		bcrash("darray_move_item: out of memory");
+		return;
+	}
+
 	p_from = darray_item(element_size, dst, from);
 	p_to = darray_item(element_size, dst, to);
 
@@ -433,6 +442,9 @@ static inline void darray_swap(const size_t element_size, struct darray *dst,
 		return;
 
 	temp = malloc(element_size);
+	if (!temp)
+		bcrash("darray_swap: out of memory");
+
 	a_ptr = darray_item(element_size, dst, a);
 	b_ptr = darray_item(element_size, dst, b);
 
@@ -482,33 +494,124 @@ static inline void darray_swap(const size_t element_size, struct darray *dst,
 
 #define da_move(dst, src) darray_move(&dst.da, &src.da)
 
-#define da_find(v, item, idx) darray_find(sizeof(*v.array), &v.da, item, idx)
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#ifdef __cplusplus
+#define da_type_test(v, item)                 \
+	({                                    \
+		if (false) {                  \
+			auto _t = v.array;    \
+			_t = (item);          \
+			(void)_t;             \
+			*(v).array = *(item); \
+		}                             \
+	})
+#else
+#define da_type_test(v, item)                       \
+	({                                          \
+		if (false) {                        \
+			const typeof(*v.array) *_t; \
+			_t = (item);                \
+			(void)_t;                   \
+			*(v).array = *(item);       \
+		}                                   \
+	})
+#endif
+#endif // ENABLE_DARRAY_TYPE_TEST
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_find(v, item, idx)                                    \
+	({                                                       \
+		da_type_test(v, item);                           \
+		darray_find(sizeof(*v.array), &v.da, item, idx); \
+	})
+#else
+#define da_find(v, item, idx) darray_find(sizeof(*v.array), &v.da, item, idx)
+#endif
+
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_push_back(v, item)                                    \
+	({                                                       \
+		da_type_test(v, item);                           \
+		darray_push_back(sizeof(*v.array), &v.da, item); \
+	})
+#else
 #define da_push_back(v, item) darray_push_back(sizeof(*v.array), &v.da, item)
+#endif
 
 #define da_push_back_new(v) darray_push_back_new(sizeof(*v.array), &v.da)
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_push_back_array(dst, src_array, n)                                  \
+	({                                                                     \
+		da_type_test(dst, src_array);                                  \
+		darray_push_back_array(sizeof(*dst.array), &dst.da, src_array, \
+				       n);                                     \
+	})
+#else
 #define da_push_back_array(dst, src_array, n) \
 	darray_push_back_array(sizeof(*dst.array), &dst.da, src_array, n)
+#endif
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_push_back_da(dst, src)                                              \
+	({                                                                     \
+		da_type_test(dst, src.array);                                  \
+		darray_push_back_darray(sizeof(*dst.array), &dst.da, &src.da); \
+	})
+#else
 #define da_push_back_da(dst, src) \
 	darray_push_back_darray(sizeof(*dst.array), &dst.da, &src.da)
+#endif
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_insert(v, idx, item)                                    \
+	({                                                         \
+		da_type_test(v, item);                             \
+		darray_insert(sizeof(*v.array), &v.da, idx, item); \
+	})
+#else
 #define da_insert(v, idx, item) \
 	darray_insert(sizeof(*v.array), &v.da, idx, item)
+#endif
 
 #define da_insert_new(v, idx) darray_insert_new(sizeof(*v.array), &v.da, idx)
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_insert_array(dst, idx, src_array, n)                       \
+	({                                                            \
+		da_type_test(dst, src_array);                         \
+		darray_insert_array(sizeof(*dst.array), &dst.da, idx, \
+				    src_array, n);                    \
+	})
+#else
 #define da_insert_array(dst, idx, src_array, n) \
 	darray_insert_array(sizeof(*dst.array), &dst.da, idx, src_array, n)
+#endif
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_insert_da(dst, idx, src)                                    \
+	({                                                             \
+		da_type_test(dst, src.array);                          \
+		darray_insert_darray(sizeof(*dst.array), &dst.da, idx, \
+				     &src.da);                         \
+	})
+#else
 #define da_insert_da(dst, idx, src) \
 	darray_insert_darray(sizeof(*dst.array), &dst.da, idx, &src.da)
+#endif
 
 #define da_erase(dst, idx) darray_erase(sizeof(*dst.array), &dst.da, idx)
 
+#ifdef ENABLE_DARRAY_TYPE_TEST
+#define da_erase_item(dst, item)                                      \
+	({                                                            \
+		da_type_test(dst, item);                              \
+		darray_erase_item(sizeof(*dst.array), &dst.da, item); \
+	})
+#else
 #define da_erase_item(dst, item) \
 	darray_erase_item(sizeof(*dst.array), &dst.da, item)
+#endif
 
 #define da_erase_range(dst, from, to) \
 	darray_erase_range(sizeof(*dst.array), &dst.da, from, to)
