@@ -5,6 +5,7 @@
 
 #include <obs-module.h>
 #include "external/nvEncodeAPI.h"
+#include "jim-nvenc-ver.h"
 
 typedef NVENCSTATUS(NVENCAPI *NV_CREATE_INSTANCE_FUNC)(
 	NV_ENCODE_API_FUNCTION_LIST *);
@@ -13,6 +14,12 @@ extern const char *nv_error_name(NVENCSTATUS err);
 extern NV_ENCODE_API_FUNCTION_LIST nv;
 extern NV_CREATE_INSTANCE_FUNC nv_create_instance;
 extern bool init_nvenc(obs_encoder_t *encoder);
-bool nv_fail(obs_encoder_t *encoder, const char *format, ...);
-bool nv_failed(obs_encoder_t *encoder, NVENCSTATUS err, const char *func,
-	       const char *call);
+bool nv_fail2(obs_encoder_t *encoder, void *session, const char *format, ...);
+bool nv_failed2(obs_encoder_t *encoder, void *session, NVENCSTATUS err,
+		const char *func, const char *call);
+
+#define nv_fail(encoder, format, ...) \
+	nv_fail2(encoder, enc->session, format, ##__VA_ARGS__)
+
+#define nv_failed(encoder, err, func, call) \
+	nv_failed2(encoder, enc->session, err, func, call)
