@@ -152,10 +152,8 @@ static bool open_video_codec(struct ffmpeg_data *data)
 	data->vframe->color_primaries = data->config.color_primaries;
 	data->vframe->color_trc = data->config.color_trc;
 	data->vframe->colorspace = data->config.colorspace;
-	data->vframe->chroma_location =
-		(data->config.colorspace == AVCOL_SPC_BT2020_NCL)
-			? AVCHROMA_LOC_TOPLEFT
-			: AVCHROMA_LOC_LEFT;
+	data->vframe->chroma_location = determine_chroma_location(
+		context->pix_fmt, data->config.colorspace);
 
 	ret = av_frame_get_buffer(data->vframe, base_get_alignment());
 	if (ret < 0) {
@@ -243,8 +241,10 @@ static bool create_video_stream(struct ffmpeg_data *data)
 
 	closest_format = data->config.format;
 	if (data->vcodec->pix_fmts) {
+		const int has_alpha = closest_format == AV_PIX_FMT_BGRA;
 		closest_format = avcodec_find_best_pix_fmt_of_list(
-			data->vcodec->pix_fmts, data->config.format, 0, NULL);
+			data->vcodec->pix_fmts, closest_format, has_alpha,
+			NULL);
 	}
 
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
@@ -262,10 +262,8 @@ static bool create_video_stream(struct ffmpeg_data *data)
 	context->color_primaries = data->config.color_primaries;
 	context->color_trc = data->config.color_trc;
 	context->colorspace = data->config.colorspace;
-	context->chroma_sample_location =
-		(data->config.colorspace == AVCOL_SPC_BT2020_NCL)
-			? AVCHROMA_LOC_TOPLEFT
-			: AVCHROMA_LOC_LEFT;
+	context->chroma_sample_location = determine_chroma_location(
+		closest_format, data->config.colorspace);
 	context->thread_count = 0;
 
 	data->video->time_base = context->time_base;
@@ -294,6 +292,7 @@ static bool open_audio_codec(struct ffmpeg_data *data, int idx)
 	AVCodecContext *const context = data->audio_infos[idx].ctx;
 	char **opts = strlist_split(data->config.audio_settings, ' ', false);
 	int ret;
+	int channels;
 
 	if (opts) {
 		parse_params(context, opts);
@@ -308,11 +307,13 @@ static bool open_audio_codec(struct ffmpeg_data *data, int idx)
 	}
 
 	data->aframe[idx]->format = context->sample_fmt;
-	data->aframe[idx]->channels = context->channels;
 #if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 24, 100)
+	data->aframe[idx]->channels = context->channels;
 	data->aframe[idx]->channel_layout = context->channel_layout;
+	channels = context->channels;
 #else
 	data->aframe[idx]->ch_layout = context->ch_layout;
+	channels = context->ch_layout.nb_channels;
 #endif
 	data->aframe[idx]->sample_rate = context->sample_rate;
 	context->strict_std_compliance = -2;
@@ -327,7 +328,7 @@ static bool open_audio_codec(struct ffmpeg_data *data, int idx)
 
 	data->frame_size = context->frame_size ? context->frame_size : 1024;
 
-	ret = av_samples_alloc(data->samples[idx], NULL, context->channels,
+	ret = av_samples_alloc(data->samples[idx], NULL, channels,
 			       data->frame_size, context->sample_fmt, 0);
 	if (ret < 0) {
 		ffmpeg_log_error(LOG_WARNING, data,
@@ -349,6 +350,7 @@ static bool create_audio_stream(struct ffmpeg_data *data, int idx)
 	AVCodecContext *context;
 	AVStream *stream;
 	struct obs_audio_info aoi;
+	int channels;
 
 	if (!obs_get_audio_info(&aoi)) {
 		ffmpeg_log_error(LOG_WARNING, data, "No active audio");
@@ -367,17 +369,20 @@ static bool create_audio_stream(struct ffmpeg_data *data, int idx)
 #endif
 	context->bit_rate = (int64_t)data->config.audio_bitrate * 1000;
 	context->time_base = (AVRational){1, aoi.samples_per_sec};
+#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 24, 100)
 	context->channels = get_audio_channels(aoi.speakers);
+#endif
+	channels = get_audio_channels(aoi.speakers);
+
 	context->sample_rate = aoi.samples_per_sec;
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(59, 24, 100)
-	context->channel_layout =
-		av_get_default_channel_layout(context->channels);
+	context->channel_layout = av_get_default_channel_layout(channels);
 
 	//avutil default channel layout for 5 channels is 5.0 ; fix for 4.1
 	if (aoi.speakers == SPEAKERS_4POINT1)
 		context->channel_layout = av_get_channel_layout("4.1");
 #else
-	av_channel_layout_default(&context->ch_layout, context->channels);
+	av_channel_layout_default(&context->ch_layout, channels);
 	if (aoi.speakers == SPEAKERS_4POINT1)
 		context->ch_layout = (AVChannelLayout)AV_CHANNEL_LAYOUT_4POINT1;
 #endif
@@ -637,9 +642,7 @@ bool ffmpeg_data_init(struct ffmpeg_data *data, struct ffmpeg_cfg *config)
 	}
 #else
 	if (is_rtmp) {
-		data->config.audio_encoder = "aac";
 		data->config.audio_encoder_id = AV_CODEC_ID_AAC;
-		data->config.video_encoder = "libx264";
 		data->config.video_encoder_id = AV_CODEC_ID_H264;
 	}
 #endif
@@ -865,14 +868,20 @@ static void encode_audio(struct ffmpeg_output *output, int idx,
 
 	AVPacket *packet = NULL;
 	int ret, got_packet;
-	size_t total_size = data->frame_size * block_size * context->channels;
+	int channels;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 24, 100)
+	channels = context->ch_layout.nb_channels;
+#else
+	channels = context->channels;
+#endif
+	size_t total_size = data->frame_size * block_size * channels;
 
 	data->aframe[idx]->nb_samples = data->frame_size;
 	data->aframe[idx]->pts = av_rescale_q(
 		data->total_samples[idx], (AVRational){1, context->sample_rate},
 		context->time_base);
 
-	ret = avcodec_fill_audio_frame(data->aframe[idx], context->channels,
+	ret = avcodec_fill_audio_frame(data->aframe[idx], channels,
 				       context->sample_fmt,
 				       data->samples[idx][0], (int)total_size,
 				       1);
