@@ -5,6 +5,7 @@
 
 /* clang-format off */
 
+#define SETTING_SDR_ONLY_INFO          "sdr_only_info"
 #define SETTING_OPACITY                "opacity"
 #define SETTING_CONTRAST               "contrast"
 #define SETTING_BRIGHTNESS             "brightness"
@@ -14,6 +15,7 @@
 #define SETTING_SIMILARITY             "similarity"
 #define SETTING_SMOOTHNESS             "smoothness"
 
+#define TEXT_SDR_ONLY_INFO             obs_module_text("SdrOnlyInfo")
 #define TEXT_OPACITY                   obs_module_text("Opacity")
 #define TEXT_CONTRAST                  obs_module_text("Contrast")
 #define TEXT_BRIGHTNESS                obs_module_text("Brightness")
@@ -321,28 +323,50 @@ static void color_key_render_v1(void *data, gs_effect_t *effect)
 
 static void color_key_render_v2(void *data, gs_effect_t *effect)
 {
+	UNUSED_PARAMETER(effect);
+
 	struct color_key_filter_data_v2 *filter = data;
 
-	if (!obs_source_process_filter_begin(filter->context, GS_RGBA,
-					     OBS_ALLOW_DIRECT_RENDERING))
-		return;
+	const enum gs_color_space preferred_spaces[] = {
+		GS_CS_SRGB,
+		GS_CS_SRGB_16F,
+		GS_CS_709_EXTENDED,
+	};
 
-	gs_effect_set_float(filter->opacity_param, filter->opacity);
-	gs_effect_set_float(filter->contrast_param, filter->contrast);
-	gs_effect_set_float(filter->brightness_param, filter->brightness);
-	gs_effect_set_float(filter->gamma_param, filter->gamma);
-	gs_effect_set_vec4(filter->key_color_param, &filter->key_color);
-	gs_effect_set_float(filter->similarity_param, filter->similarity);
-	gs_effect_set_float(filter->smoothness_param, filter->smoothness);
+	const enum gs_color_space source_space = obs_source_get_color_space(
+		obs_filter_get_target(filter->context),
+		OBS_COUNTOF(preferred_spaces), preferred_spaces);
+	if (source_space == GS_CS_709_EXTENDED) {
+		obs_source_skip_video_filter(filter->context);
+	} else {
+		const enum gs_color_format format =
+			gs_get_format_from_space(source_space);
+		if (obs_source_process_filter_begin_with_color_space(
+			    filter->context, format, source_space,
+			    OBS_ALLOW_DIRECT_RENDERING)) {
+			gs_effect_set_float(filter->opacity_param,
+					    filter->opacity);
+			gs_effect_set_float(filter->contrast_param,
+					    filter->contrast);
+			gs_effect_set_float(filter->brightness_param,
+					    filter->brightness);
+			gs_effect_set_float(filter->gamma_param, filter->gamma);
+			gs_effect_set_vec4(filter->key_color_param,
+					   &filter->key_color);
+			gs_effect_set_float(filter->similarity_param,
+					    filter->similarity);
+			gs_effect_set_float(filter->smoothness_param,
+					    filter->smoothness);
 
-	gs_blend_state_push();
-	gs_blend_function(GS_BLEND_ONE, GS_BLEND_INVSRCALPHA);
+			gs_blend_state_push();
+			gs_blend_function(GS_BLEND_ONE, GS_BLEND_INVSRCALPHA);
 
-	obs_source_process_filter_end(filter->context, filter->effect, 0, 0);
+			obs_source_process_filter_end(filter->context,
+						      filter->effect, 0, 0);
 
-	gs_blend_state_pop();
-
-	UNUSED_PARAMETER(effect);
+			gs_blend_state_pop();
+		}
+	}
 }
 
 static bool key_type_changed(obs_properties_t *props, obs_property_t *p,
@@ -398,6 +422,9 @@ static obs_properties_t *color_key_properties_v2(void *data)
 {
 	obs_properties_t *props = obs_properties_create();
 
+	obs_properties_add_text(props, SETTING_SDR_ONLY_INFO,
+				TEXT_SDR_ONLY_INFO, OBS_TEXT_INFO);
+
 	obs_property_t *p = obs_properties_add_list(props, SETTING_COLOR_TYPE,
 						    TEXT_COLOR_TYPE,
 						    OBS_COMBO_TYPE_LIST,
@@ -454,6 +481,24 @@ static void color_key_defaults_v2(obs_data_t *settings)
 	obs_data_set_default_int(settings, SETTING_SMOOTHNESS, 50);
 }
 
+static enum gs_color_space
+color_key_get_color_space(void *data, size_t count,
+			  const enum gs_color_space *preferred_spaces)
+{
+	const enum gs_color_space potential_spaces[] = {
+		GS_CS_SRGB,
+		GS_CS_SRGB_16F,
+		GS_CS_709_EXTENDED,
+	};
+
+	struct color_key_filter_data_v2 *const filter = data;
+	const enum gs_color_space source_space = obs_source_get_color_space(
+		obs_filter_get_target(filter->context),
+		OBS_COUNTOF(potential_spaces), potential_spaces);
+
+	return source_space;
+}
+
 struct obs_source_info color_key_filter = {
 	.id = "color_key_filter",
 	.type = OBS_SOURCE_TYPE_FILTER,
@@ -479,4 +524,5 @@ struct obs_source_info color_key_filter_v2 = {
 	.update = color_key_update_v2,
 	.get_properties = color_key_properties_v2,
 	.get_defaults = color_key_defaults_v2,
+	.video_get_color_space = color_key_get_color_space,
 };
