@@ -11,6 +11,7 @@ extern struct obs_source_info mask_filter;
 extern struct obs_source_info mask_filter_v2;
 extern struct obs_source_info crop_filter;
 extern struct obs_source_info gain_filter;
+extern struct obs_source_info eq_filter;
 extern struct obs_source_info hdr_tonemap_filter;
 extern struct obs_source_info color_filter;
 extern struct obs_source_info color_filter_v2;
@@ -36,6 +37,7 @@ extern struct obs_source_info noise_gate_filter;
 extern struct obs_source_info compressor_filter;
 extern struct obs_source_info limiter_filter;
 extern struct obs_source_info expander_filter;
+extern struct obs_source_info upward_compressor_filter;
 extern struct obs_source_info luma_key_filter;
 extern struct obs_source_info luma_key_filter_v2;
 #ifdef LIBNVVFX_ENABLED
@@ -50,6 +52,7 @@ bool obs_module_load(void)
 	obs_register_source(&mask_filter_v2);
 	obs_register_source(&crop_filter);
 	obs_register_source(&gain_filter);
+	obs_register_source(&eq_filter);
 	obs_register_source(&hdr_tonemap_filter);
 	obs_register_source(&color_filter);
 	obs_register_source(&color_filter_v2);
@@ -77,6 +80,7 @@ bool obs_module_load(void)
 	obs_register_source(&compressor_filter);
 	obs_register_source(&limiter_filter);
 	obs_register_source(&expander_filter);
+	obs_register_source(&upward_compressor_filter);
 	obs_register_source(&luma_key_filter);
 	obs_register_source(&luma_key_filter_v2);
 #ifdef LIBNVVFX_ENABLED
