@@ -1,7 +1,5 @@
 #include <obs-module.h>
 
-#include "obs-outputs-config.h"
-
 #ifdef _WIN32
 #include <winsock2.h>
 #include <mbedtls/threading.h>
@@ -17,7 +15,7 @@ MODULE_EXPORT const char *obs_module_description(void)
 extern struct obs_output_info rtmp_output_info;
 extern struct obs_output_info null_output_info;
 extern struct obs_output_info flv_output_info;
-#if COMPILE_FTL
+#if defined(FTL_FOUND)
 extern struct obs_output_info ftl_output_info;
 #endif
 
@@ -67,7 +65,7 @@ bool obs_module_load(void)
 	obs_register_output(&rtmp_output_info);
 	obs_register_output(&null_output_info);
 	obs_register_output(&flv_output_info);
-#if COMPILE_FTL
+#if defined(FTL_FOUND)
 	obs_register_output(&ftl_output_info);
 #endif
 	return true;
